#version 300 es

precision mediump float;

out vec4 FragColor;

struct LightProperties
{
	vec3 direction;
	vec4 ambient;
	vec4 diffuse;
	vec4 specular;
};

struct MaterialProperties
{
	vec4 ambient;
	vec4 diffuse;
	vec4 specular;
	vec4 baseColor;
	float shininess;
	int type;
};


uniform	MaterialProperties material;
uniform vec3 viewPos;

in vec3 FragPos;
in vec3 Normal;
in vec2 TexCoords;
in vec3 LightPos;
in vec4 diffuseLight;

uniform	LightProperties dirLight_1;
uniform	LightProperties dirLight_2;

#define ALL_LIGHTS 8
uniform LightProperties allLight[ALL_LIGHTS];

#define NR_POINT_LIGHTS 4
uniform LightProperties dirLights[NR_POINT_LIGHTS];

//uniform PointLight pointLights[NR_POINT_LIGHTS];


vec4 CalcDirLight1(LightProperties light, vec3 normal, vec3 viewDir);
vec4 CalcDirLight(LightProperties light, vec3 normal, vec3 viewDir);

void main()
{
    vec3 norm = normalize(Normal);
    vec3 viewDir = normalize(viewPos - FragPos);
    vec4 result = vec4(0.0,0.0,0.0,0.0);

    if(material.type==0){
        vec3 halfVector = normalize(-LightPos);
        float NdotH = max(dot(norm, halfVector), 0.0);
        float specular = NdotH;
        vec4 colorSpecular = vec4(material.specular.xyz * specular, 1.0);
        result = diffuseLight*0.5+(material.ambient+material.baseColor)*0.7 ;
        for(int i = 0; i < 8; i++){
         result += CalcDirLight(allLight[i], norm,viewDir);
        }
        result=vec4(result.rgb,0.85);
    }
    if(material.type>=1){
            result +=CalcDirLight(dirLight_1, norm, viewDir);
            result += CalcDirLight(dirLight_2, norm, viewDir);
            for(int i = 0; i < NR_POINT_LIGHTS; i++){
               if(material.type>1){
                    result += CalcDirLight(dirLights[i], norm,viewDir);
               }
            }
    }
    FragColor = result;
}

vec4 CalcDirLight1(LightProperties light, vec3 normal, vec3 viewDir)
{
    vec3 lightDir = normalize(-light.direction);
    // diffuse shading
    float diff = max(dot(normal, lightDir), 0.0);
    // specular shading
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
    // combine results
    vec4 ambient = light.ambient * material.diffuse*material.baseColor;
    vec4 diffuse = light.diffuse * diff * material.diffuse*material.baseColor;
    vec4 specular = light.specular * spec * material.specular*material.baseColor;
    return (ambient + diffuse);
}

vec4 CalcDirLight(LightProperties light, vec3 normal, vec3 viewDir)
{
    vec3 lightDir = normalize(-light.direction);
    // diffuse shading
    float diff = max(dot(normal, lightDir), 0.0);
    // specular shading
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
    // combine results
    vec4 ambient = light.ambient * material.diffuse*material.baseColor;
    vec4 diffuse = light.diffuse * diff * material.diffuse*material.baseColor;
    vec4 specular = light.specular * spec * material.specular*material.baseColor;
    return (ambient + diffuse + specular);
}
