#version 300 es

//#extension GL_EXT_texture_buffer : enabled

precision mediump float;
out vec4 FragColor;

//layout (binding = 0, r32ui) uniform uimage2D head_pointer_image;
//layout (binding = 1, rgba32ui) uniform uimageBuffer list_buffer;
//layout (binding = 0, offset = 0) uniform atomic_uint list_counter;


struct LightProperties
{
	vec3 direction;
	vec4 ambient;
	vec4 diffuse;
	vec4 specular;
};

struct MaterialProperties
{
	vec4 ambient;
	vec4 diffuse;
	vec4 specular;
	vec4 baseColor;
	float shininess;
	int type;
};


uniform	MaterialProperties material;
uniform vec3 viewPos;

in vec3 FragPos;
in vec3 Normal;
in vec2 TexCoords;
in vec3 LightPos;
in vec4 diffuseLight;
in vec3 WorldPos;


uniform	LightProperties dirLight_1;
uniform	LightProperties dirLight_2;

#define ALL_LIGHTS 8
uniform LightProperties allLight[ALL_LIGHTS];

#define NR_POINT_LIGHTS 4
uniform LightProperties dirLights[NR_POINT_LIGHTS];

uniform samplerCube prefilterMap;
//uniform sampler2D brdfLUT;


vec3 fresnelSchlickRoughness(float cosTheta, vec3 F0, float roughness);

vec4 CalcDirLight1(LightProperties light, vec3 normal, vec3 viewDir);
vec4 CalcDirLight(LightProperties light, vec3 normal, vec3 viewDir);

void main()
{
    vec3 norm = normalize(Normal);
    vec3 viewDir = normalize(viewPos - FragPos);

    vec4 result = vec4(0.0,0.0,0.0,0.0);

    if(material.type==0){
        vec3 halfVector = normalize(-LightPos);
        float NdotH = max(dot(normalize(Normal), halfVector), 0.0);
        float specular = NdotH;
        vec4 colorSpecular = material.specular * specular;
        result =  diffuseLight*0.6+colorSpecular+(material.ambient+material.baseColor)*0.9;
        result=vec4(result.rgb,0.85);
    }

    if(material.type>=1){
        if(material.type==10){
                 vec3 cameraPos = vec3(0.0,100.0,0.0);
                 vec3 I = normalize(cameraPos-WorldPos);
                 vec3 R = reflect(-I, Normal);
                 result = vec4(texture(prefilterMap, R).rgb, 1.0);
        }else{
           // result +=CalcDirLight(dirLight_1, norm, viewDir);
           // result += CalcDirLight(dirLight_2, norm, viewDir);

            result +=CalcDirLight(dirLight_1, norm, viewDir);
            result += CalcDirLight(dirLight_2, norm, viewDir);
            result = vec4(result.rgb*vec3(2.6),1.0);

            //for(int i = 0; i < NR_POINT_LIGHTS; i++){
               // if(material.type>1){
                   // result += CalcDirLight(dirLights[i], norm,viewDir);
               // }
           // }
        }

    }

    FragColor = result;
}

vec4 CalcDirLight1(LightProperties light, vec3 normal, vec3 viewDir)
{
    vec3 lightDir = normalize(-light.direction);
    // diffuse shading
    float diff = max(dot(normal, lightDir), 0.0);
    // specular shading
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
    // combine results
    vec4 ambient = light.ambient * material.diffuse*material.baseColor;
    vec4 diffuse = light.diffuse * diff * material.diffuse*material.baseColor;
    vec4 specular = light.specular * spec * material.specular*material.baseColor;
    return (ambient + diffuse);
}

/**
vec4 CalcDirLight1(LightProperties light, vec3 normal, vec3 viewDir)
{
    float ambientStrength = 0.3;    //环境因子
    float specularStrength = 2.0;   //镜面强度
    float reflectance = 256.0;      //反射率
    vec3 paraLightDir = normalize(-light.direction);
    vec4 ambient = ambientStrength * material.baseColor;
    vec3 norm = normalize(Normal);
    vec3 lightDir = normalize(light.direction - FragPos);
    float diff = max(dot(norm,paraLightDir),0.0);
    vec4 diffuse = diff * light.diffuse * material.baseColor;
    vec3 viewDir = normalize(viewPos - FragPos);
    vec3 reflectDir = reflect(-paraLightDir,Normal);
    float spec = pow(max(dot(viewDir, reflectDir),0.0),reflectance);
    vec4 specular = specularStrength * spec * material.baseColor;
    //距离衰减常量
    float constantPara = 1.0;
    //线性衰减常量
    float linearPara = 0.09;
    //二次衰减常量
    float quadraticPara = 0.032;
    //衰减因子计算
    float LFDistance = length(light.direction - FragPos);
    //衰减因子 =  1.0/(距离衰减常量 + 线性衰减常量 * 距离 + 二次衰减常量 * 距离的平方)
    float lightWeakPara = 1.0/(constantPara + linearPara * LFDistance + quadraticPara * (LFDistance*LFDistance));

    //光照颜色 =(环境颜色 + 漫反射颜色 + 镜面反射颜色)* 衰减因子
    vec4 res = (ambient + diffuse + specular)*lightWeakPara;
    return res;
}
*/
// calculates the color when using a directional light.
vec4 CalcDirLight(LightProperties light, vec3 normal, vec3 viewDir)
{
    vec3 lightDir = normalize(-light.direction);
    // diffuse shading
    float diff = max(dot(normal, lightDir), 0.0);
    // specular shading
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
    // combine results
    vec4 ambient = light.ambient * material.diffuse*material.baseColor;
    vec4 diffuse = light.diffuse * diff * material.diffuse*material.baseColor;
    vec4 specular = light.specular * spec * material.specular*material.baseColor;
    return (ambient + diffuse + specular);
}


vec3 fresnelSchlickRoughness(float cosTheta, vec3 F0, float roughness)
{
    return F0 + (max(vec3(1.0 - roughness), F0) - F0) * pow(max(1.0 - cosTheta, 0.0), 5.0);
}

